'=========================================================================
' Tastaturgestuetztes textbasiertes User Interface fuer die Konsole
' Erstellt 14.07.06
' Letzte Ueberarbeitung am 13.11.13
'=========================================================================


'Wichtige Tastaturabfragecodes, die von GETKEY zurueckgegeben werden:
Const SC_BKSP = &H08, SC_TAB = &H09, SC_SHIFT_TAB = &H0FFF, SC_ENTER = &H0D, _
	SC_ESC = &H1B, SC_LEFT = &H4BFF, SC_RIGHT = &H4DFF, SC_UP = &H48FF, SC_DOWN = &H50FF, _
	SC_INS = &H52FF, SC_DEL = &H53FF, _
	SC_END = &H4FFF, SC_HOME = &H47FF, SC_PGUP = &H49FF, SC_PGDOWN = &H51FF, _
  SC_F1 = &H3BFF, SC_F2 = &H3CFF, SC_F3 = &H3DFF, SC_F4 = &H3EFF, SC_F5 = &H3FFF, _
  SC_F6 = &H40FF, SC_F7 = &H41FF, SC_F8 = &H42FF, SC_F9 = &H43FF, SC_F10 = &H44FF, _
  SC_F11 = &H85FF, SC_F12 = &H86FF


Sub Zentriert(Zeile As Integer, Text As String)
' Zentrierter Text

  Dim As Integer Breite

  Breite = Loword(Width)
  Locate Zeile, (Breite - Len(Text)) \ 2
  Print Text

End Sub


Sub ClScreen(o As Integer, l As Integer, u As Integer, r As Integer)
' Loescht einen Bildschirmausschnitt durch Ueberschreiben mit der
' Hintergrundfarbe.
' - o  = Position oberer Rand
' - l  = Position linker Rand
' - u  = Position unterer Rand
' - r  = Position rechter Rand

  Dim As Integer Breite, y

  Breite = r - l + 1
  For y = o To u
    Locate y, l: Print Space(Breite);
  Next y

End Sub


Sub Rahmen(Titel As String, Fusszeile As String, o As Integer, l As Integer, _
  u As Integer, r As Integer)
' Zeichnet einen Rahmen mit Titel.
' - Titel = Titel zentriert (falls Titel <> "")
' - Fusszeile = Fusszeile zentriert (falls Fusszeile <> "")
' - o  = Position oberer Rand
' - l  = Position linker Rand
' - u  = Position unterer Rand
' - r  = Position rechter Rand
' Der Rahmen wird zum Loeschen des Hintergrundes mit aktuellen Hintergrundfarbe
' gefuellt. Erfolgt vorher eine entsprechende COLOR-Anweisung, wird ein farbiges
' Fenster mit Rahmen gezeichnet.

  Dim As Integer Breite, Hoehe, y
  Dim As String*1 eol, eor, eul, eur, hlin, vlin

  eol = Chr(218)
  eor = Chr(191)
  eul = Chr(192)
  eur = Chr(217)
  hlin = Chr(196)
  vlin = Chr(179)

  ' Hintergrund loeschen:
  ClScreen(o, l, u, r)

  ' Rahmen zeichnen:
  Locate o, l: Print eol; String(r - l - 1, hlin); eor;
  For y = o + 1 To u - 1
    Locate y, l: Print vlin;
    Locate y, r: Print vlin;
  Next y
  Locate u, l: Print eul; String(r - l - 1, hlin); eur;

  ' Titel und Fusszeile einfuegen:
  Breite = r - l + 1
  If Titel <> "" Then
    Locate o, l + (Breite - Len(Titel)) \ 2
    Print Titel;
  End If
  If Fusszeile <> "" Then
    Locate u, l + (Breite - Len(Titel)) \ 2
    Print Fusszeile;
  End If

End Sub


Sub InString(Byref s As String, Laenge As Integer, Byref ExitCode As Integer, _
  Mark As Integer = 1)
' Prozedur zum Editieren eines String mit Vorbelegung und laengenbegrenztem
' Eingabefeld. Parameter:
' - s = zu editierender String
' - Laenge = Laenge des Eingabefeldes. Wenn LEN(s) > Laenge, wird s im Eingabefeld
'   horizontal gescrollt.
' - ExitCode = Tastencode, mit dem die SUB verlassen wurde (SC_ENTER, SC_ESC,
'   SC_UP, SC_DOWN, SC_TAB, SC_SHIFT_TAB)
' - Mark <> 0: Eingabefeld wird farbig markiert.
  
  Dim As Integer Zeile, Spalte, fgcolor, bkcolor, key, CPos, Offset, row, col
  
  Zeile = Csrlin
  Spalte = Pos
  If Mark <> 0 Then
  	fgcolor = Loword(Color)
  	bkcolor = Hiword(Color)
  	If bkcolor = 7 Then Color 0, 15 Else Color 0,7
    EndIf
    
    CPos = 0
    Offset = 0
        
    'Tastaturabfrage und -auswertung
    Do
      
      'Eingabebalken mit String:
      Locate Zeile, Spalte: Print Space(Laenge);
      Locate Zeile, Spalte: Print Mid(s, Offset + 1, Laenge);
      Locate Zeile, Spalte + CPos - Offset
                  
      'Warten bis Tastaturpuffer leer, dann Tasten abfragen: 
      Do
        Sleep(1)
      Loop Until Inkey = ""
      key = Getkey
                  
      Select Case key
      Case SC_ENTER, SC_ESC, SC_UP, SC_DOWN, SC_TAB, SC_SHIFT_TAB
        'Ende
        ExitCode = key
        Exit Do 
      Case SC_LEFT
        'Eine Position nach links
        If CPos > 0 Then 
          CPos = CPos - 1
          If CPos < Offset Then Offset = Offset - 1
        End If
      Case SC_RIGHT
        'Eine Position nach rechts
        If CPos < Len(s) Then
          CPos = CPos + 1
          If CPos > Laenge - 1 + Offset Then Offset = Offset + 1
        End If
      Case SC_HOME
        'Zum Anfang
        CPos = 0
        Offset = 0
      Case SC_END
        'Zum Ende
        CPos = Len(s)
        If Len(s) > Laenge - 1 Then Offset = Len(s) - Laenge + 1 Else Offset = 0 
      Case SC_DEL
        If (Len(s) > 0) And (CPos < Len(s)) Then _
        s = Left(s, CPos) + Right(s, Len(s) - CPos - 1)
      Case SC_BKSP
        If (Len(s) > 0) And (CPos > 0) Then
          s = Left(s, CPos - 1) + Right(s, (Len(s) - CPos))
          CPos = CPos - 1
          If Offset > 0 Then Offset = Offset - 1
        End If
      Case 32 To 255  
        'druckbare Zeichen
        s = Left(s, CPos) + Chr(key) + Right(s, Len(s) - CPos)
        CPos = CPos + 1
        If CPos > Laenge - 1 + Offset Then Offset = Offset + 1
      End Select
      
    Loop
    
    'Ende:
    If Mark Then Color(fgcolor, bkcolor)
    Locate Zeile, Spalte: Print Space(Laenge);
    Locate Zeile, Spalte: Print Left(s, Laenge);
    
End Sub


Function Edit(s As String, Laenge As Integer = Loword(Width) - Pos - 1, _
  Mark As Integer = 1) As String
' Editieren eines Strings (Ersatz fuer die INPUT-Anweisung). 
' Beenden mit ENTER. Parameter Laenge und Mark (optional) siehe SUB InString.

  Dim As Integer Zeile, Spalte, ExitCode
  Dim As String s1 

  Zeile = Csrlin
  Spalte = Pos
  Do
    Locate Zeile, Spalte
    s1 = s
    InString(s1, Laenge, ExitCode, Mark)
  Loop Until ExitCode = SC_ENTER
  Print 'neue Zeile
  
  Return s1

End Function


Sub EditList (s() As String, Laenge As Integer = Loword(Width) - Pos - 1, Mark As Integer = 1)
' Editieren einer Variablenliste s(0) bis s(n). Kann auch als "Editor" verwendet
' werden. Die obere linke Ecke des Eingabefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden (gilt auch fuer die nachfolgenden Prozeduren).
' Parameter:
' - s(0) bis s(n) = Textzeilen
' - Laenge und Mark siehe SUB InString
' Verlassen der Prozedur erfolgt mit Esc.

  Dim As Integer Zeile, Spalte, i, n, ExitCode

  Zeile = Csrlin
  Spalte = Pos
  n = Ubound(s)

  For i = 0 To n
    Locate Zeile + i, Spalte: Print s(i);
  Next i

  i = 0
  Do
    Locate Zeile + i, Spalte
    InString(s(i), Laenge, ExitCode, Mark)
    Select Case ExitCode
    Case SC_DOWN, SC_ENTER, SC_TAB
      'Eine Zeile weiter:
      If i < n Then i = i + 1 Else i = 0
    Case SC_UP, SC_SHIFT_TAB
      'Eine Zeile zurueck:
      If i > 0 Then i = i - 1 Else i = n
    Case SC_ESC
      'Ende:
      Exit Do
    End Select
  Loop

End Sub


Sub EditTab(s() As String, SpBreite() As Integer, Mark As Integer = 1)
' Tabelle editieren. Parameter:
' - s(0, 0) bis s(m, n) = Variablen (Zeile 0 bis m, Spalte 0 bis n)
' - SpBreite(0) bis SpBreite(n) = Spaltenbreiten (entspricht dem Parameter Laenge
'   fuer jede Variable, siehe SUB InString)
' - Mark: siehe Sub Instring
' Die linke obere Ecke des Eingabefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden.
' Verlassen der Prozedur erfolgt mit Esc.

  Dim As Integer Zeile, Spalte, m, n, i, j, ExitCode

  Zeile = Csrlin
  Spalte = Pos
  m = Ubound(s, 1)
  n = Ubound(s, 2)
  Dim As Integer col(n)
  
  'Berechnung der Spaltenpositionen:
  col(0) = Spalte
  For j = 1 To n
    col(j) = col(j - 1) + SpBreite(j - 1)
  Next j
  
  For i = 0 To m
    For j = 0 To n
      Locate Zeile + i, col(j)
      Print s(i, j)
    Next j
  Next i

  i = 0
  j = 0
  Do
    Locate Zeile + i, col(j): InString(s(i, j), SpBreite(j) - 1, ExitCode, Mark)
    Select Case ExitCode
    Case SC_ENTER
      'Zeile fuer Zeile von einer Spalte zur naechsten:
      If j < n Then 
        j = j + 1 
      Elseif i < m Then
        i = i + 1
        j = 0
      Else
        i = 0
        j = 0
      End If
    Case SC_TAB: If j < n Then j = j + 1 Else j = 0
    Case SC_SHIFT_TAB: If j > 0 Then j = j - 1 Else j = n
    Case SC_UP: If i > 0 Then i = i - 1 Else i = m
    Case SC_DOWN: If i < m Then i = i + 1 Else i = 0
    Case SC_ESC: Exit Do
    End Select
  Loop

End Sub


Sub Eingabe(Maske() As String, s() As String, Laenge As Integer, Mark As Integer = 1)
' Eingabemaske. Parameter:
' - Maske(0) bis Maske(n) = Eingabemaske (muessen alle gleich lang sein, z.B.
'   mit Leerzeichen auffuellen).
' - s(0) bis s(n) = zu editierende Variablen
' - Laenge(0) bis Laenge(n) = maximale Eingabelaenge fuer jede Variable
' - Mark: siehe Sub Instring
' Die linke obere Ecke des Eingabefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden.
' Verlassen erfolgt mit ESC.

  Dim As Integer Zeile, Spalte, i, n, col

  Zeile = Csrlin
  Spalte = Pos
  n = Ubound(s)

  'Maske:
  For i = 0 To n
    Locate Zeile + i, Spalte: Print Maske(i);
    col = Pos
  Next i

  'Variablen anzeigen und editieren:
  
  Locate Zeile, col
  EditList(s(), Laenge, Mark)

End Sub


Sub Menue(s() As String, Byref sNr As Integer)
' Menue. Parameter:
' - s(0) bis s(n) = Menue-Eintraege (sollten alle gleich lang sein, z.B. mit
'   Leerzeichen auffuellen)
' - sNr = Nummer des gewaehlten Menue-Eintrages
' Die linke obere Ecke des Menuefeldes kann durch eine LOCATE-Anweisung
' festgelegt werden.

  Dim As Integer Zeile, Spalte, fgcolor, bkcolor, i, j, n, key

  fgcolor = Loword(Color)
  bkcolor = Hiword(Color) 
  Zeile = Csrlin
  Spalte = Pos
  n = Ubound(s)
  
  i = 0
  Do
    
    For j = 0 To n
      If j = i Then 
        If bkcolor = 7 Then Color 15, 3 Else Color 0,7
      Else
        Color fgcolor, bkcolor
      End If
      Locate Zeile + j, Spalte, 0	'Cursor aus
      Print s(j);
    Next j
    
    key = Getkey
    Select Case key
      Case SC_DOWN
        If i < n Then i = i + 1 Else i = 0
      Case SC_UP
        If i > 0 Then i = i - 1 Else i = n
      Case SC_ENTER
        Exit Do          
    End Select

  Loop
  
  SNr = i
  Locate ,,1  'Cursor wieder ein
  
End Sub
